@@ -396,6 +396,7 @@ polhensellift(GEN pol, GEN L, GEN p, long N)
   GEN T = NULL;
   long i, l, t;
   pari_sp av = avma;
+  void (*chk)(GEN, const char*);
 
   if (typ(pol) != t_POL) pari_err_TYPE("polhensellift",pol);
   RgX_check_ZXX(pol, "polhensellift");
@@ -408,15 +409,16 @@ polhensellift(GEN pol, GEN L, GEN p, long N)
     RgX_check_ZX(T, "polhensellift");
     p = gel(p,1); t = typ(p);
   }
+  chk = T? RgX_check_ZXX: RgX_check_ZX;
   if (t != t_INT) pari_err_TYPE("polhensellift",p);
   if (N < 1) pari_err_DOMAIN("polhensellift", "precision", "<", gen_1,stoi(N));
 
   l = lg(L); L = leafcopy(L);
   for (i = 1; i < l; i++)
   {
-    if (typ(gel(L,i)) != t_POL)
-      gel(L,i) = scalar_ZX_shallow(gel(L,i), varn(pol));
-    RgX_check_ZXX(gel(L,i), "polhensellift");
+    GEN q = gel(L,i);
+    if (typ(q) != t_POL) gel(L,i) = scalar_ZX_shallow(q, varn(pol));
+    else chk(q, "polhensellift");
   }
   return gerepilecopy(av, ZqX_liftfact(pol, L, T, powiu(p,N), p, N));
 }
@@ -555,11 +557,12 @@ ZpXQX_liftroot_vald(GEN f, GEN a, long v, GEN T, GEN p, long e)
     fr = FpXQX_red(f, Tq, qv);
     fa = FqX_eval(fr, a, Tq, qv);
     fa = typ(fa)==t_INT? diviiexact(fa,q2v): ZX_Z_divexact(fa, q2v);
-    a = Fq_sub(a, ZX_Z_mul(Fq_mul(W, fa, Tq2, q2v), q2), Tq, qv);
+    a = Fq_sub(a, Fq_mul(Fq_mul(W,fa,Tq2,q2v),q2, Tq,qv), Tq, qv);
     if (mask == 1) return gerepileupto(av, a);
     dfr = FpXQX_red(df, Tq, q);
-    u = ZX_Z_divexact(FpX_Fp_sub(Fq_mul(W,FqX_eval(dfr,a,Tq,q),Tq,q),gen_1,q),q2);
-    W = Fq_sub(W,ZX_Z_mul(Fq_mul(u,W,Tq2,q2),q2),Tq,q);
+    u = Fq_sub(Fq_mul(W,FqX_eval(dfr,a,Tq,q),Tq,q),gen_1,Tq,q);
+    u = typ(u)==t_INT? diviiexact(u,q2): ZX_Z_divexact(u,q2);
+    W = Fq_sub(W, Fq_mul(Fq_mul(u,W,Tq2,q2),q2, Tq,q),Tq,q);
     if (gc_needed(av2,2))
     {
       if(DEBUGMEM>1) pari_warn(warnmem,"ZpXQX_liftroot, e = %ld", e);
