@@ -1938,10 +1938,12 @@ all_roots(GEN p, long bit)
 {
   GEN lc, pd, q, roots_pol, m;
   long bit0,  bit2, i, e, h, n = degpol(p);
+  double fb;
   pari_sp av;
 
   pd = RgX_deflate_max(p, &h); lc = leading_coeff(pd);
-  e = (long)(2 * fujiwara_bound(pd)); if (e < 0) e = 0;
+  fb = fujiwara_bound(pd);
+  e = (fb < 0)? 0: (long)(2 * fb);
   bit0 = bit + gexpo(pd) - gexpo(lc) + (long)log2(n/h)+1+e;
   bit2 = bit0; e = 0;
   for (av=avma,i=1;; i++,avma=av)
@@ -2184,14 +2186,16 @@ GEN
 QX_complex_roots(GEN p, long l)
 {
   pari_sp av = avma;
-  long bit;
+  long bit, v;
   GEN L;
 
   if (!signe(p)) pari_err_ROOTS0("QX_complex_roots");
   if (lg(p) == 3) return cgetg(1,t_COL); /* constant polynomial */
   if (l < LOWDEFAULTPREC) l = LOWDEFAULTPREC;
   bit = prec2nbits(l);
-  L = all_roots(Q_primpart(p), bit);
+  v = RgX_valrem(p, &p);
+  L = lg(p) > 3? all_roots(Q_primpart(p), bit): cgetg(1,t_COL);
+  if (v) L = shallowconcat(const_vec(v, real_0_bit(-bit)), L);
   return gerepileupto(av, clean_roots(L, l, bit, 1));
 }
 
