@@ -560,7 +560,7 @@ powuu(ulong p, ulong N)
   if (N <= 2)
   {
     if (N == 2) return sqru(p);
-    if (N == 1) return utoipos(p);
+    if (N == 1) return utoi(p);
     return gen_1;
   }
   if (!p) return gen_0;
@@ -1908,10 +1908,18 @@ exp1r_abs(GEN x)
 GEN
 mpexpm1(GEN x)
 {
-  long sx = signe(x);
+  const long s = 6;
+  long l, sx = signe(x);
   GEN y, z;
   pari_sp av;
   if (!sx) return real_0_bit(expo(x));
+  l = realprec(x);
+  if (l > maxss(EXPNEWTON_LIMIT, (1L<<s) + 2))
+  {
+    long e = expo(x);
+    if (e < 0) x = rtor(x, l + nbits2extraprec(-e));
+    return subrs(mpexp(x), 1);
+  }
   if (sx > 0) return exp1r_abs(x);
   /* compute exp(x) * (1 - exp(-x)) */
   av = avma; y = exp1r_abs(x);
@@ -3321,8 +3329,9 @@ gsinc(GEN x, long prec)
     case t_COMPLEX:
       if (isintzero(gel(x,1)))
       {
-        av = avma;
-        return gerepileuptoleaf(av,gdiv(gsinh(gel(x,2),prec),gel(x,2)));
+        av = avma; x = gel(x,2);
+        if (gequal0(x)) return gcosh(x,prec);
+        return gerepileuptoleaf(av,gdiv(gsinh(x,prec),x));
       }
       i = precision(x); if (i) prec = i;
       y = cgetc(prec); av = avma;
