@@ -418,8 +418,18 @@ mpsinh(GEN x)
 
   if (!signe(x)) return real_0_bit(ex);
   lx = realprec(x); res = cgetr(lx); av = avma;
-  if (ex < 1 - BITS_IN_LONG) x = rtor(x, lx + nbits2extraprec(-ex)-1);
-  z = mpexp(x); z = subrr(z, invr(z)); shiftr_inplace(z, -1);
+  if (ex < 1 - BITS_IN_LONG)
+  { /* y = e^x-1; e^x - e^(-x) = y(1 + 1/(y+1)) */
+    GEN y = mpexpm1(x);
+    z = addrs(y,1); if (lg(z) > lx+1) z = rtor(z,lx+1); /* e^x */
+    z = mulrr(y, addsr(1,invr(z)));
+  }
+  else
+  {
+    z = mpexp(x);
+    z = subrr(z, invr(z));
+  }
+  shiftr_inplace(z, -1);
   affrr(z, res); avma = av; return res;
 }
 
@@ -1183,7 +1193,8 @@ cxgamma(GEN s0, int dolog, long prec)
     l2 = u*u - st*st;
     if (l2 > 0)
     {
-      nn = (long)ceil(sqrt(l2) - ssig);
+      double t = ceil(sqrt(l2) - ssig);
+      nn = (t < 1)? 1: (long)t;
       if (nn < 1) nn = 1;
     }
     else
@@ -1517,11 +1528,38 @@ serlngamma0(GEN y, long prec)
   return integser(t);
 }
 
+static GEN
+serlngamma(GEN y, long prec)
+{
+  GEN z, y0, Y;
+  if (lg(y) == 2) pari_err_DOMAIN("gamma", "argument", "=", gen_0,y);
+  /* exp(lngamma) */
+  if (valp(y) > 0) return gdiv(gexp(glngamma(gaddgs(y,1),prec),prec),y);
+  y0 = simplify_shallow(gel(y,2));
+  z = NULL; Y = y;
+  if (isint(y0, &y0))
+  { /* fun eq. avoids log singularity of lngamma at negative ints */
+    long s = signe(y0);
+    /* possible if y[2] is an inexact 0 */
+    if (!s) return gdiv(gexp(glngamma(gaddgs(y,1),prec),prec),y);
+    if (signe(y0) < 0) { Y = gsubsg(1, y); y0 = subsi(1, y0); }
+    if (abscmpiu(y0, 50) < 0) z = mpfact(itos(y0)-1); /* more precise */
+  }
+  if (!z) z = ggamma(y0,prec);
+  z = gmul(z, gexp(serlngamma0(Y,prec),prec));
+  if (Y != y)
+  {
+    GEN pi = mppi(prec);
+    z = gdiv(mpodd(y0)? pi: negr(pi),
+             gmul(z, gsin(gmul(pi,serchop0(y)), prec)));
+  }
+  return z;
+}
 GEN
 ggamma(GEN x, long prec)
 {
   pari_sp av;
-  GEN y, z;
+  GEN y;
 
   switch(typ(x))
   {
@@ -1575,29 +1613,7 @@ ggamma(GEN x, long prec)
     case t_PADIC: return Qp_gamma(x);
     default:
       av = avma; if (!(y = toser_i(x))) break;
-      if (lg(y) == 2) pari_err_DOMAIN("gamma", "argument", "=", gen_0,y);
-      /* exp(lngamma) */
-      if (valp(y) > 0 || lg(y) == 2)
-        z = gdiv(gexp(glngamma(gaddgs(y,1),prec),prec),y);
-      else
-      {
-        GEN Y = y, y0 = simplify_shallow(gel(y,2));
-        z = NULL;
-        if (isint(y0, &y0))
-        { /* fun eq. avoids log singularity of lngamma at negative ints */
-          if (signe(y0) < 0) { Y = gsubsg(1, y); y0 = subsi(1, y0); }
-          if (abscmpiu(y0, 50) < 0) z = mpfact(itos(y0)-1); /* more precise */
-        }
-        if (!z) z = ggamma(y0,prec);
-        z = gmul(z, gexp(serlngamma0(Y,prec),prec));
-        if (Y != y)
-        {
-          GEN pi = mppi(prec);
-          z = gdiv(mpodd(y0)? pi: negr(pi),
-                   gmul(z, gsin(gmul(pi,serchop0(y)), prec)));
-        }
-      }
-      return gerepileupto(av, z);
+      return gerepileupto(av, serlngamma(y, prec));
   }
   return trans_eval("gamma",ggamma,x,prec);
 }
