@@ -16,6 +16,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA. */
 #include "anal.h"
 #include "opcode.h"
 
+INLINE void
+clone_unlock_deep(GEN C) { if (isclone(C)) gunclone_deep(C); }
+
 /********************************************************************/
 /*                                                                  */
 /*                   break/next/return handling                     */
@@ -735,8 +738,12 @@ get_next_label(const char *s, int member, char **next_fun)
 static const char *
 get_arg_name(GEN C, long i)
 {
-  GEN e = gmael(closure_get_dbg(C), 3, 1);
-  return ((entree*)e[i])->name;
+  GEN d = closure_get_dbg(C), frpc = gel(d,2), fram = gel(d,3);
+  long j, l = lg(frpc);
+  for (j=1; j<l; j++)
+    if (frpc[j]==1 && i<lg(gel(fram,j)))
+      return ((entree*)mael(fram,j,i))->name;
+  return "(unnamed)";
 }
 
 void
@@ -824,7 +831,7 @@ ptr_proplock(gp_pointer *g, GEN C)
   g->x = C;
   if (isclone(g->x))
   {
-    clone_unlock(g->ox);
+    clone_unlock_deep(g->ox);
     g->ox = g->x;
     ++bl_refc(g->ox);
   }
@@ -976,7 +983,7 @@ closure_eval(GEN C)
             changevalue(g->ep, g->x);
         }
         else change_compo(&(g->c), g->x);
-        clone_unlock(g->ox);
+        clone_unlock_deep(g->ox);
       }
       break;
     case OCstoredyn:
@@ -993,7 +1000,7 @@ closure_eval(GEN C)
       {
         gp_pointer *g = &ptrs[--rp];
         change_compo(&(g->c), gel(st,--sp));
-        clone_unlock(g->ox);
+        clone_unlock_deep(g->ox);
         break;
       }
     case OCstackgen:
@@ -1483,7 +1490,7 @@ endeval:
     for(  ; rp>saved_rp ;  )
     {
       gp_pointer *g = &ptrs[--rp];
-      clone_unlock(g->ox);
+      clone_unlock_deep(g->ox);
     }
   }
   s_prec.n = saved_prec;
